# -*- coding: utf-8 -*-
from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
from django.db.models import loading
from shutil import rmtree
from django.conf import settings
from os.path import join
import sys

def progressbar(it, prefix = "", size = 60):
    count = len(it)
    def _show(_i):
        x = int(size*_i/count)
        sys.stdout.write("%s[%s%s] %i/%i\r" % (prefix, "#"*x, "."*(size-x), _i, count))
        sys.stdout.flush()

    _show(0)
    for i, item in enumerate(it):
        yield item
        _show(i+1)
    sys.stdout.write("\n")
    sys.stdout.flush()


class Command(BaseCommand):
    args = 'applabel.Model'
    help = '''Regeneartes djnago-imagekit cached images.
    Usage: manage.py imagekit_gencache attachment.AttachemntImage [--force]
    '''
    option_list = BaseCommand.option_list + (
        make_option('--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force delete cache directory'),
    )

    def handle(self, *args, **options):
        app_label, model_name =args[0].split('.')
        model_cls = loading.get_model(app_label, model_name)
        ikspecs = model_cls._ik.specs
        
        if options['force']:
            # Delete cache dir
            print 'Forced remove previous cache dir: ', model_cls._ik.cache_dir,
            try:
                rmtree(join(settings.MEDIA_ROOT, model_cls._ik.cache_dir), ignore_errors=False)
            except (IOError, OSError):
                print '[Fail]'
            else:
                print '[OK]'

        for spec in ikspecs:
            for instance in progressbar(model_cls.objects.all(), "Pre-caching %s: " % spec.name(), 80):
                img = getattr(instance, spec.name())
                getattr(img, 'url')



from redsolutioncms.make import BaseMake
from redsolutioncms.models import CMSSettings

class Make(BaseMake):
    def make(self):
        super(Make, self).make()
        cms_settings = CMSSettings.objects.get_settings()
        cms_settings.render_to('settings.py', 'utilities/redsolutioncms/settings.pyt')
        cms_settings.render_to('development.py', 'utilities/redsolutioncms/development.pyt')
        cms_settings.render_to(['..', 'templates', 'base_utilities.html'],
            'utilities/redsolutioncms/base_utilities.html', {
        }, 'w')
        cms_settings.base_template = 'base_utilities.html'
        cms_settings.save()

make = Make()




# -*- coding: utf-8 -*-
from django import forms
from django.contrib import admin
from django.contrib.sessions.models import Session
from django.http import HttpResponseRedirect
from django.contrib.auth.forms import UserChangeForm, PasswordResetForm
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from utilities.settings import ALLOW_DOT_IS_USERNAME, ENABLE_PASSWORD_RESET
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext, ugettext_lazy as _
from django.shortcuts import render_to_response, get_object_or_404
from django.utils.html import escape
from django.template import RequestContext
try:
    from django.contrib import messages
except ImportError:
    pass

class MoidfiedPasswordResetForm(PasswordResetForm):

    def clean_email(self):
        """
        Validates that a user exists with the given e-mail address.
        Closes Django ticket #14674 http://code.djangoproject.com/ticket/14674
        Also resets all user's sesions and sets unusable password for user
        """
        from django.contrib.auth.models import UNUSABLE_PASSWORD
        email = self.cleaned_data["email"]
        self.users_cache = User.objects.filter(email__iexact=email).exclude(
            password=UNUSABLE_PASSWORD)
        if len(self.users_cache) == 0:
            raise forms.ValidationError(_("That e-mail address doesn't have an associated user account. Are you sure you've registered?"))
        return email

    def save(self, *args, **kwds):
        # Now clear user's sessions:
        for session in Session.objects.all():
            for user in self.users_cache:
                user.set_unusable_password()
                user.save()
                if session.get_decoded().get('_auth_user_id') == user.id:
                    session.delete()

        super(MoidfiedPasswordResetForm, self).save(*args, **kwds)

class ModifiedUserChangeForm(UserChangeForm):
    email = forms.EmailField(label=_("E-mail"), max_length=75,
        help_text=_('<a href="%s">Reset password</a> for this user.' % 'reset/'),
        required=False)

class ResetPasswordMixin(object):
    '''
        Mixin adds reset password link to user change form in admin site.
        Added view with ResetPasswordForm, user will set temporary password
        and recieve email with reset password link. 
    '''

    def __init__(self, *args, **kwds):
        super(ResetPasswordMixin, self).__init__(*args, **kwds)
        self.form = ModifiedUserChangeForm


    def get_urls(self):
        from django.conf.urls.defaults import patterns
        return patterns('',
            (r'^(\d+)/reset/$', self.admin_site.admin_view(self.reset_password))
        ) + super(ResetPasswordMixin, self).get_urls()

    def reset_password(self, request, id):
        '''
        Almost copied from ``django.contrb.auth.admin.UserAdmin.user_change_password``
        Resets user password to teporary value, send email to user with reset-password link.
        '''
        if not self.has_change_permission(request):
            raise PermissionDenied
        user = get_object_or_404(self.model, pk=id)
        if request.method == 'POST':
            form = MoidfiedPasswordResetForm(request.POST)
            if form.is_valid():
                new_user = form.save()
                try:
                    msg = ugettext('Password for user %s has been reseted.' % user)
                    messages.success(request, msg)
                except NameError:
                    # there's no messages framework in Django 1.1
                    pass
                return HttpResponseRedirect('..')
        else:
            msg = ugettext('Password for user %s has been reseted.' % user)
            form = MoidfiedPasswordResetForm({'email': user.email})

        fieldsets = [(None, {'fields': form.base_fields.keys()})]
        adminForm = admin.helpers.AdminForm(form, fieldsets, {})

        return render_to_response(self.change_user_password_template or 'admin/reset_password.html', {
            'title': _('Reset password: %s') % escape(user.username),
            'adminForm': adminForm,
            'form': form,
            'is_popup': '_popup' in request.REQUEST,
            'add': True,
            'change': False,
            'has_delete_permission': False,
            'has_change_permission': True,
            'has_absolute_url': False,
            'opts': self.model._meta,
            'original': user,
            'save_as': False,
            'show_save': True,
            'root_path': self.admin_site.root_path,
        }, context_instance=RequestContext(request))


if ENABLE_PASSWORD_RESET:
    UserAdminClass = admin.site._registry[User].__class__

    class NewClass(ResetPasswordMixin, UserAdminClass):
        pass

    try:
        admin.site.unregister(User)
    except admin.sites.NotRegistered:
        raise ImproperlyConfigured("Can not found User admin. Check if 'utilities' inserted in INSTALLED_APPS *after* 'django.contrib.auth'")

    admin.site.register(User, NewClass)



# -*- coding: utf-8 -*-
from datetime import date, timedelta, datetime
from django import template
from django.utils import dateformat
from django.utils.translation import ugettext_lazy as _

MONTHS = [
    _(' January'), _(' February'), _(' March'), _(' April'), _(' May'),
    _(' June'), _(' July'), _(' August'), _(' September'), _(' October'),
    _(' November'), _(' December')
]

register = template.Library()


class DateFormatLocal(dateformat.DateFormat):

    def F(self):
        "Month, textual, long; e.g. 'January'"
        return MONTHS[self.data.month - 1]


def format(value, format_string):
    "Convenience function"
    df = DateFormatLocal(value)
    return df.format(format_string)

@register.filter(name='date_local')
def date_local(value, arg=None):
    """Formats a date according to the given format."""
    if not value:
        return u''
    if arg is None:
        arg = settings.DATE_FORMAT
    try:
        return format(value, arg)
    except AttributeError:
        return ''
date_local.is_safe = False



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *
from utilities import settings

urlpatterns = patterns('',)

if settings.ENABLE_PASSWORD_RESET:
    urlpatterns += patterns('',
        (r'^reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', 'django.contrib.auth.views.password_reset_confirm'),
        (r'^reset/done/$', 'django.contrib.auth.views.password_reset_complete',
            {'template_name': 'admin/password_reset_complete.html'}),
    )



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.management import create_permissions
from django.db.models import get_apps


class Command(BaseCommand):
    help = 'Update permissions for all INSTALLED_APPS'

    def handle(self, *args, **options):
        for app in get_apps():
            create_permissions(app, None, 0)




import re
from django.contrib.auth.forms import UserCreationForm, UserChangeForm
from utilities.settings import ALLOW_DOT_IS_USERNAME

if ALLOW_DOT_IS_USERNAME:
    UserCreationForm.base_fields['username'].regex = re.compile(r'^[-\.\w]+$')
    UserChangeForm.base_fields['username'].regex = re.compile(r'^[-\.\w]+$')



from functools import wraps
from django.template.context import Context
from django.shortcuts import render_to_response

def private_context(f):
    """
    Source at: http://djangosnippets.org/snippets/1687/
    
    Simple decorator which avoids the need to a) copy-and-paste code to force
    context variables into inclusion_tag templates and b) carefully avoid
    inclusion tag variables conflicting with global variables.
    
    Instead each inclusion tag will be called with a *copy* of the provided
    context variable and its results will be merged in to avoid leaking into
    the global context
    """

    @wraps(f)
    def private_context_wrapper(context, *args, **kwargs):
        c = Context(context)
        rc = f(c, *args, **kwargs)
        c.update(rc)
        return c

    return private_context_wrapper


def render_to(template):
    """
    Source at: http://djangosnippets.org/snippets/821/
    
    Decorator for Django views that sends returned dict to render_to_response function
    with given template and RequestContext as context instance.

    If view doesn't return dict then decorator simply returns output.
    Additionally view can return two-tuple, which must contain dict as first
    element and string with template name as second. This string will
    override template name, given as parameter

    Parameters:

     - template: template name to use
    """
    @wraps(func)
    def wrapper(request, *args, **kw):
        output = func(request, *args, **kw)
        if isinstance(output, (list, tuple)):
            return render_to_response(output[1], output[0], RequestContext(request))
        elif isinstance(output, dict):
            return render_to_response(template, output, RequestContext(request))
        return output
    return wrapper



# -*- coding: utf-8 -*-

from django.utils.safestring import mark_safe
from django.contrib.admin.widgets import AdminFileWidget

class ImagePreviewWidget(AdminFileWidget):
    def __init__(self, thumb_field=None, thumb_size=(80, 80), *args, **kwargs):
        """
        ``thumb_field`` name of field with thumb image. If None will use image itself.
        ``thumb_size`` maximum (width, height) to be displayed. If None original size will be used.
        """
        self.thumb_field = thumb_field
        self.thumb_size = thumb_size
        super(ImagePreviewWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        output = []
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        instance = getattr(value, 'instance', None)
        if instance is not None:
            try:
                field = getattr(instance, value.field.name, None)
                if field:
                    if self.thumb_field is None:
                        thumb_field = field
                    else:
                        thumb_field = getattr(instance, self.thumb_field, None)
                    if self.thumb_size is None:
                        style = ''
                    else:
                        cur_width, cur_height = field.width, field.height
                        thumb_width, thumb_height = self.thumb_size
                        if not thumb_width is None and not thumb_height is None:
                            ratio = min(float(thumb_width) / cur_width,
                                        float(thumb_height) / cur_height)
                        else:
                            if thumb_width is None:
                                ratio = float(thumb_height) / cur_height
                            else:
                                ratio = float(thumb_width) / cur_width
                        new_size = (int(round(cur_width * ratio)), int(round(cur_height * ratio)))
                        if new_size[0] > cur_width or new_size[1] > cur_height:
                            new_size = self.thumb_size
                        style = 'style="width: %spx; height: %spx; "' % new_size
                    output.append('<br /><a target="_blank" href="%s"><img src="%s" alt="%s" %s/></a>' % \
                        (field.url, thumb_field.url, field, style))
            except IOError:
                output = ['<p class="errornote">' + _('File access error') + '</p>',] + output
        return mark_safe(u''.join(output))



# -*- coding: utf-8 -*-

from django import forms
from django.db import models
from django.contrib.admin.widgets import AdminFileWidget
from django.utils import dates
from utilities.widgets import ImagePreviewWidget
import datetime

class ImagePreviewFormField(forms.ImageField):
    widget = ImagePreviewWidget

class ImagePreviewField(models.ImageField):
    def __init__(self, thumb_field=None, thumb_size=(80, 80), *args, **kwargs):
        """
        ``thumb_field`` name of field with thumb image. If None will use image itself.
        ``thumb_size`` maximum (width, height) to be displayed. If None original size will be used.
        """
        self.thumb_field = thumb_field
        self.thumb_size = thumb_size
        super(ImagePreviewField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'widget': ImagePreviewWidget}
        defaults.update(kwargs)

        # As an ugly hack, we override the admin widget
        if defaults['widget'] == AdminFileWidget:
            defaults['widget'] = ImagePreviewWidget

        if defaults['widget'] == ImagePreviewWidget:
            defaults['widget'] = ImagePreviewWidget(thumb_field=self.thumb_field, thumb_size=self.thumb_size)

        return super(ImagePreviewField, self).formfield(**defaults)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([
        (
            [ImagePreviewField],
            [],
            {
                "thumb_field": ["thumb_field", {"default": None}],
                "thumb_size": ["thumb_size", {"default": (80, 80)}],
            },
        ),
    ], [
        "^utilities\.fields\.ImagePreviewField",
    ])
except ImportError:
    pass

#    SplitDateField
EMPTY = [('', u'---')]
DAYS = [(day, '%02d' % day) for day in xrange(1, 32)]
MONTHS = [(month, dates.MONTHS[month]) for month in xrange(1, 13)]
DEFAULT_FROM_YEAR = 1930

def widget_factory(years):
    class SplitDateWidget(forms.MultiWidget):
        def __init__(self, attrs=None):
            widgets = (
                forms.Select(attrs=None, choices=EMPTY + DAYS),
                forms.Select(attrs=None, choices=EMPTY + MONTHS),
                forms.Select(attrs=None, choices=EMPTY + years),
            )
            super(SplitDateWidget, self).__init__(widgets, attrs)

        def decompress(self, value):
            if value:
                return [value.day, value.month, value.year]
            return [None, None, None]
    return  SplitDateWidget

class SplitDateFormField(forms.MultiValueField):

    def __init__(self, from_date=datetime.date(DEFAULT_FROM_YEAR, 01, 01),
        till_date=datetime.date.today, reverse=False, *args, **kwargs):
        if callable(from_date):
            from_date = from_date()
        if callable(till_date):
            till_date = till_date()
        self.from_date = from_date
        self.till_date = till_date
        from_year = from_date.year
        till_year = till_date.year
        years = [(year, '%04d' % year) for year in xrange(till_year,
            from_year - 1, -1)]
        if reverse:
            years.reverse()
        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])
        kwargs['widget'] = widget_factory(years)
        fields = (
            forms.ChoiceField(choices=DAYS),
            forms.ChoiceField(choices=MONTHS),
            forms.ChoiceField(choices=years),
        )
        super(SplitDateFormField, self).__init__(fields, *args, **kwargs)

    def compress(self, value_list):
        if value_list:
            for value in value_list:
                if value in forms.fields.EMPTY_VALUES:
                    raise forms.ValidationError(forms.SplitDateTimeField.default_error_messages['invalid_date'])
            kwargs = {
                'day': int(value_list[0]),
                'month': int(value_list[1]),
                'year': int(value_list[2]),
            }
            try:
                date = datetime.date(**kwargs)
                if date > self.till_date or date < self.from_date:
                    raise forms.ValidationError(forms.SplitDateTimeField.default_error_messages['invalid_date'])
                else:
                    return date
            except ValueError:
                raise forms.ValidationError(forms.SplitDateTimeField.default_error_messages['invalid_date'])
        return None

class SplitDateField(models.DateField):
    def __init__(self, from_date=datetime.date(DEFAULT_FROM_YEAR, 01, 01),
        till_date=datetime.date.today, reverse=False, *args, **kwargs):
        self.from_date = from_date
        self.till_date = till_date
        self.reverse = reverse
        super(SplitDateField, self).__init__(*args, **kwargs)

    def formfield(self, ** kwargs):
        defaults = {'form_class': SplitDateFormField, }
        defaults.update(kwargs)
        return super(SplitDateField, self).formfield(from_date=self.from_date,
            till_date=self.till_date, reverse=self.reverse, ** defaults)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^utilities\.fields\.SplitDateField"])
except ImportError:
    pass
